@@ -4,14 +4,14 @@
 #include "knightcap.h"
 #if LARGE_ETYPE
 #include "large_coeffs.h"
-#else 
+#else
 #include "small_coeffs.h"
 #endif
 
 #define EVAL_ALL_DEPTH 2
 
-#define EVAL_SHORTCUT_THRESHOLD (2.0*PAWN_VALUE)
-#define EVAL_SHORTCUT_OFFSET (1.5*PAWN_VALUE)
+#define EVAL_SHORTCUT_THRESHOLD (2.0*STATIC_PAWN_VALUE)
+#define EVAL_SHORTCUT_OFFSET (1.5*STATIC_PAWN_VALUE)
 
 /* coefficients we don't want changed by TD(lambda) (multiplicative factors, 
  the value of a pawn, the value of a king, and the value of a draw */
@@ -123,6 +123,38 @@ void update_pawns(Position *b)
 	}
 }
 
+static int get_control(Position *b, uint32 topieces,
+		       int piece, Square sq) 
+{
+	int nw, nb;
+	/* the following masks select pieces that are of lower than
+	   value than a piece */
+	static const uint32 masks[2*KING+1] = 
+	{0xFFFE, 0xFFFC, 0xFFF0, 0xFF00, 0xFF00, 0, 
+	 0, 
+	 0, 0xFF000000, 0xFF000000, 0xFFF00000, 0xFFFC0000, 0xFFFE0000};
+
+	if (!topieces) return 0;
+
+	if (topieces & masks[piece+KING]) 
+		return -piece;
+
+	/* compare pawn counts */
+	nw = pop_count[(topieces >> 8) & 0xFF];
+	nb = pop_count[(topieces >> 24) & 0xFF];
+	if (nw - nb) return nw - nb;
+
+	/* compare piece counts */
+	nw = pop_count[topieces & 0xFF];
+	nb = pop_count[(topieces >> 16) & 0xFF];
+
+#if USE_SLIDING_CONTROL
+	nw += pop_count[b->xray_topieces[sq] & 0xFF];
+	nb += pop_count[(b->xray_topieces[sq] >> 16) & 0xFF];
+#endif
+	return (nw - nb);
+}
+
 #define weak_pawn_attack_value (coefficients + IWEAK_PAWN_ATTACK_VALUE)
 /* this evaluates a pawn - its more valuable if its a passed
    pawn or on an open file. In those cases it gains value
@@ -138,6 +170,8 @@ static etype eval_white_pawn(Position *b, int pi)
 	int weak = 0;
 	Square pos;
 
+	b->wpassed_pawn_mask &= ~(1<<pi);
+
 	ret = pawn_pos_value[piece->pos];
 
 	if ((x != 0 && b->board[piece->pos+WEST] == PAWN) ||
@@ -153,6 +187,8 @@ static etype eval_white_pawn(Position *b, int pi)
 
 	/* penalize slightly unsupported pawns */
 	ret -= UNSUPPORTED_PAWN;
+	if (debug) 
+		lprintf(0,"unsupported %s\n", posstr(piece->pos));
 
 	if ((b->topieces[piece->pos+NORTH] & WPAWN_MASK) &&
 	    abs(b->board[piece->pos+NORTH]) != PAWN)
@@ -192,33 +228,78 @@ not_weak:
 		    (x==7 || black_pawn_loc[x+1] <= y)) {
 			Square wkpos = WHITEPIECES(b)[IKING].pos;
 			Square bkpos = BLACKPIECES(b)[IKING].pos;
-			int a;
+			int a, control;
+			uint32 rook_support = 0;
+			uint32 topieces;
 
 			b->wpassed_pawn_mask |= (1<<pi);
+			pos = piece->pos;
 
 			advance += pawn_advance[n];
-
-			pos = piece->pos;
-			a = passed_pawn_index[y-1];
-			while (!off_board(pos,NORTH)) {
-				pos += NORTH;
-				if (b->cont[pos] > 0) {
-					advance += passed_pawn_control[a];
-					++a;
-				} else {
-					break;
-				}
-			}
-
+			
 			if (YPOS(bkpos) >= y && abs(XPOS(bkpos) - x) <= 1)
 				advance -= KING_PASSED_PAWN_DEFENCE;
 
 			if (YPOS(wkpos) >= y && abs(XPOS(wkpos) - x) <= 1)
 				advance += KING_PASSED_PAWN_SUPPORT;
 
-			if (b->board[piece->pos+NORTH] < 0)
+			if (b->board[pos+NORTH] < 0)
 				advance -= BLOCKED_PASSED_PAWN;
 
+			if ((b->topieces[pos] & ~b->pinned_mask & BKROOK_MASK) &&
+			    XPOS(BLACKPIECES(b)[IKROOK].pos) == x &&
+			    YPOS(BLACKPIECES(b)[IKROOK].pos) < y) {
+				/* there is a black king rook 
+				   behind our passed pawn */
+				advance -= PASSED_PAWN_ROOK_ATTACK;
+			}
+
+			if ((b->topieces[pos] & ~b->pinned_mask & BQROOK_MASK) &&
+			    XPOS(BLACKPIECES(b)[IQROOK].pos) == x &&
+			    YPOS(BLACKPIECES(b)[IQROOK].pos) < y) {
+				/* there is a black queen rook 
+				   behind our passed pawn */
+				advance -= PASSED_PAWN_ROOK_ATTACK;
+			}
+
+			if ((b->topieces[pos] & ~b->pinned_mask & WKROOK_MASK) &&
+			    XPOS(WHITEPIECES(b)[IKROOK].pos) == x &&
+			    YPOS(WHITEPIECES(b)[IKROOK].pos) < y) {
+				/* there is a white king rook 
+				   behind our passed pawn */
+				rook_support = WKROOK_MASK;
+				advance += PASSED_PAWN_ROOK_SUPPORT;
+			}
+
+			if ((b->topieces[pos] & ~b->pinned_mask & WQROOK_MASK) &&
+			    XPOS(WHITEPIECES(b)[IQROOK].pos) == x &&
+			    YPOS(WHITEPIECES(b)[IQROOK].pos) < y) {
+				/* there is a white queen rook 
+				   behind our passed pawn */
+				rook_support = WQROOK_MASK;
+				advance += PASSED_PAWN_ROOK_SUPPORT;
+			}
+
+			a = passed_pawn_index[y-1];
+			while (!off_board(pos,NORTH)) {
+				pos += NORTH;
+				/* if rook_support == 1 we have to adjust the control 
+				   of the squares in front of the pawn */
+				if (!rook_support) {
+					control = b->cont[pos];
+				} else {
+					topieces = b->topieces[pos] & (~b->pinned_mask);
+					topieces |= rook_support;
+					control = get_control(b, topieces, b->board[pos], pos);
+				}
+				if (control >= 0) {
+					advance += passed_pawn_control[a];
+					++a;
+				} else {
+					break;
+				}
+			}
+
 			if (x != 0 && white_pawn_loc[x-1] &&
 			    abs(y - white_pawn_loc[x-1]) <= 1) {
 				advance += pawn_advance1[n];
@@ -229,37 +310,6 @@ not_weak:
 				advance += pawn_advance1[n];
 			}
 
-			if ((b->topieces[piece->pos] & ~b->pinned_mask & BKROOK_MASK) &&
-			    XPOS(BLACKPIECES(b)[IKROOK].pos) == x &&
-			    YPOS(BLACKPIECES(b)[IKROOK].pos) < y) {
-				/* there is a black king rook 
-				   behind our passed pawn */
-				advance -= PASSED_PAWN_ROOK_ATTACK;
-			}
-
-			if ((b->topieces[piece->pos] & ~b->pinned_mask & BQROOK_MASK) &&
-			    XPOS(BLACKPIECES(b)[IQROOK].pos) == x &&
-			    YPOS(BLACKPIECES(b)[IQROOK].pos) < y) {
-				/* there is a black queen rook 
-				   behind our passed pawn */
-				advance -= PASSED_PAWN_ROOK_ATTACK;
-			}
-
-			if ((b->topieces[piece->pos] & ~b->pinned_mask & WKROOK_MASK) &&
-			    XPOS(WHITEPIECES(b)[IKROOK].pos) == x &&
-			    YPOS(WHITEPIECES(b)[IKROOK].pos) < y) {
-				/* there is a white king rook 
-				   behind our passed pawn */
-				advance += PASSED_PAWN_ROOK_SUPPORT;
-			}
-
-			if ((b->topieces[piece->pos] & ~b->pinned_mask & WQROOK_MASK) &&
-			    XPOS(WHITEPIECES(b)[IQROOK].pos) == x &&
-			    YPOS(WHITEPIECES(b)[IQROOK].pos) < y) {
-				/* there is a white queen rook 
-				   behind our passed pawn */
-				advance += PASSED_PAWN_ROOK_SUPPORT;
-			}
 
 			/* a special case - is it unstoppable? */
 			if ((b->piece_mask & BLACK_MASK) == BKING_MASK) {
@@ -351,6 +401,8 @@ static etype eval_black_pawn(Position *b, int pi)
 	int weak = 0;
 	Square pos;
 
+	b->bpassed_pawn_mask &= ~(1<<pi);
+
 	ret = pawn_pos_value[mirror_square(piece->pos)];
 
 	/* a pawn is weak if it isn't supported by a pawn and can't
@@ -365,6 +417,8 @@ static etype eval_black_pawn(Position *b, int pi)
 		goto not_weak;
 
 	ret -= UNSUPPORTED_PAWN;
+	if (debug) 
+		lprintf(0, "unsupported %s\n", posstr(piece->pos));
 
 	if ((b->topieces[piece->pos+SOUTH] & BPAWN_MASK) &&
 	    abs(b->board[piece->pos+SOUTH]) != PAWN)
@@ -406,23 +460,15 @@ static etype eval_black_pawn(Position *b, int pi)
 		     white_pawn_loc[x+1] >= y)) {
 			Square wkpos = WHITEPIECES(b)[IKING].pos;
 			Square bkpos = BLACKPIECES(b)[IKING].pos;
-			int a;
+			int a, control;
+			uint32 rook_support = 0;
+			uint32 topieces;
 
 			b->bpassed_pawn_mask |= (1<<pi);
+			pos = piece->pos;
 
 			advance += pawn_advance[n];
 
-			pos = piece->pos;
-			a = passed_pawn_index[7-y-1];
-			while (!off_board(pos,SOUTH)) {
-				pos += SOUTH;
-				if (b->cont[pos] < 0) {
-					advance += passed_pawn_control[a];
-					++a;
-				} else {
-					break;
-				}
-			}
 
 			if (YPOS(wkpos) <= y && abs(XPOS(wkpos) - x) <= 1)
 				advance -= KING_PASSED_PAWN_DEFENCE;
@@ -466,6 +512,7 @@ static etype eval_black_pawn(Position *b, int pi)
 			    YPOS(BLACKPIECES(b)[IKROOK].pos) > y) {
 				/* there is a black king rook 
 				   behind our passed pawn */
+				rook_support = BKROOK_MASK;
 				advance += PASSED_PAWN_ROOK_SUPPORT;
 			}
 
@@ -474,10 +521,29 @@ static etype eval_black_pawn(Position *b, int pi)
 			    YPOS(BLACKPIECES(b)[IQROOK].pos) > y) {
 				/* there is a black queen rook 
 				   behind our passed pawn */
+				rook_support = BQROOK_MASK;
 				advance += PASSED_PAWN_ROOK_SUPPORT;
 			}
 
 
+			a = passed_pawn_index[7-y-1];
+			while (!off_board(pos,SOUTH)) {
+				pos += SOUTH;
+				if (!rook_support) {
+					control = b->cont[pos];
+				} else {
+					topieces = b->topieces[pos] & (~b->pinned_mask);
+					topieces |= rook_support;
+					control = get_control(b, topieces, b->board[pos], pos);
+				}
+				if (control <= 0) {
+					advance += passed_pawn_control[a];
+					++a;
+				} else {
+					break;
+				}
+			}
+
 			/* a special case - is it unstoppable? */
 			if ((b->piece_mask & WHITE_MASK) == WKING_MASK) {
 				int kpos = WHITEPIECES(b)[IKING].pos;
@@ -579,7 +645,7 @@ static etype eval_white_king(Position *b, int pi)
 		}
 	} else {
 		if (b->stage == MATING) {
-			if (b->w_material < b->b_material - PAWN_VALUE) {
+			if (b->w_material < b->b_material - STATIC_PAWN_VALUE) {
 				ret += 10*(ending_kpos[XPOS(piece->pos)] +
 					   ending_kpos[YPOS(piece->pos)]);
 			} else {
@@ -662,7 +728,7 @@ static etype eval_black_king(Position *b, int pi)
 		}
 	} else {
 		if (b->stage == MATING) {
-			if (b->b_material < b->w_material - PAWN_VALUE) {
+			if (b->b_material < b->w_material - STATIC_PAWN_VALUE) {
 				ret += 10*(ending_kpos[XPOS(piece->pos)] +
 					   ending_kpos[YPOS(piece->pos)]);
 			} else {
@@ -993,14 +1059,14 @@ static etype specifics(Position *b)
 	/* opposite bishops is drawish */
 	if ((b->piece_mask & BISHOP_MASK) == (WKBISHOP_MASK | BKBISHOP_MASK) ||
 	    (b->piece_mask & BISHOP_MASK) == (WQBISHOP_MASK | BQBISHOP_MASK)) {
-		if (b->b_material > b->w_material + PAWN_VALUE/2)
+		if (b->b_material > b->w_material + STATIC_PAWN_VALUE/2)
 			ret += OPPOSITE_BISHOPS;
-		else if (b->w_material > b->b_material + PAWN_VALUE/2)
+		else if (b->w_material > b->b_material + STATIC_PAWN_VALUE/2)
 			ret -= OPPOSITE_BISHOPS;		
 	}
 
 	/* these are some specialist endgame things */
-	if (b->w_material < b->b_material - PAWN_VALUE &&
+	if (b->w_material < b->b_material - STATIC_PAWN_VALUE &&
 	    b->stage == MATING) {
 		int kpos = WHITEPIECES(b)[IKING].pos;
 
@@ -1014,7 +1080,7 @@ static etype specifics(Position *b)
 		}
 	}
 
-	if (b->b_material < b->w_material - PAWN_VALUE &&
+	if (b->b_material < b->w_material - STATIC_PAWN_VALUE &&
 	    b->stage == MATING) {
 		int kpos = BLACKPIECES(b)[IKING].pos;
 
@@ -1092,37 +1158,6 @@ static void recalc_pos_values(Position *b)
 /* this basically tries to answer the question "can one of the players
    profitably capture on that square, assuming there was something there
    to capture */
-static int get_control(Position *b, uint32 topieces,
-		       int piece, Square sq) 
-{
-	int nw, nb;
-	/* the following masks select pieces that are of lower than
-	   value than a piece */
-	static const uint32 masks[2*KING+1] = 
-	{0xFFFE, 0xFFFC, 0xFFF0, 0xFF00, 0xFF00, 0, 
-	 0, 
-	 0, 0xFF000000, 0xFF000000, 0xFFF00000, 0xFFFC0000, 0xFFFE0000};
-
-	if (!topieces) return 0;
-
-	if (topieces & masks[piece+KING]) 
-		return -piece;
-
-	/* compare pawn counts */
-	nw = pop_count[(topieces >> 8) & 0xFF];
-	nb = pop_count[(topieces >> 24) & 0xFF];
-	if (nw - nb) return nw - nb;
-
-	/* compare piece counts */
-	nw = pop_count[topieces & 0xFF];
-	nb = pop_count[(topieces >> 16) & 0xFF];
-
-#if USE_SLIDING_CONTROL
-	nw += pop_count[b->xray_topieces[sq] & 0xFF];
-	nb += pop_count[(b->xray_topieces[sq] >> 16) & 0xFF];
-#endif
-	return (nw - nb);
-}
 
 static etype board_control1(Position *b)
 {
@@ -1433,7 +1468,6 @@ static etype mobility_fn(Position *b, int pi)
 	Piece p = abs(b->pieces[pi].p);
 	Square pos = b->pieces[pi].pos;
 	int y = YPOS(b->pieces[pi].pos);
-
 	etype *mobility_table[KING+1] = {NULL, NULL,
 					         &coefficients[IKNIGHT_MOBILITY],
 						 &coefficients[IBISHOP_MOBILITY],
@@ -1448,6 +1482,9 @@ static etype mobility_fn(Position *b, int pi)
 		 &coefficients[IQUEEN_SMOBILITY],
 		 &coefficients[IKING_SMOBILITY]};
 
+	if (p != b->pieces[pi].p)
+		y = 7-y;
+
 	mobility = b->mobility[pi];
 	if (mobility >= 10) {
 		mobility = 9;
@@ -1632,6 +1669,41 @@ static etype eval_mobility(Position *b)
 	return total;
 }
 
+int egtb(Position *b) 
+{
+	int ret = 0;
+
+#if USE_EGTB
+	if (pop_count32(b->material_mask) < 5 || 
+	    (pop_count32(b->material_mask) == 5 && 
+	     ((b->w_material == KING_VALUE + ROOK_VALUE + KNIGHT_VALUE &&
+	       b->b_material == KING_VALUE + ROOK_VALUE) || 
+	      (b->b_material == KING_VALUE + ROOK_VALUE + KNIGHT_VALUE &&
+	       b->w_material == KING_VALUE + ROOK_VALUE) ||
+	      (b->w_material == KING_VALUE + ROOK_VALUE + BISHOP_VALUE &&
+	       b->b_material == KING_VALUE + ROOK_VALUE) || 
+	      (b->b_material == KING_VALUE + ROOK_VALUE + BISHOP_VALUE &&
+	       b->w_material == KING_VALUE + ROOK_VALUE))))
+		ret = 1;
+#endif
+	return ret;
+}
+		
+
+int no_material(Position *b) 
+{
+
+	if (((b->material_mask & WHITE_MASK) == (b->piece_mask & WHITE_MASK) && 
+	     (b->w_material <= emax(KING_VALUE+BISHOP_VALUE, KING_VALUE+KNIGHT_VALUE) ||
+	      b->w_material == KING_VALUE+2*KNIGHT_VALUE)) &&
+	    ((b->material_mask & BLACK_MASK) == (b->piece_mask & BLACK_MASK) && 
+	     (b->b_material <= emax(KING_VALUE+BISHOP_VALUE, KING_VALUE+KNIGHT_VALUE) ||
+	      b->b_material == KING_VALUE+2*KNIGHT_VALUE))) {
+		return 1;
+	}
+	
+	return 0;
+}	
 
 #define piece_trade_bonus (coefficients + IPIECE_TRADE_BONUS)
 #define pawn_trade_bonus (coefficients + IPAWN_TRADE_BONUS)
@@ -1642,8 +1714,34 @@ static etype check_material(Position *b, etype ret0)
 {
 	etype ret = ret0;
 	int num = 0;
-	etype vdraw = draw_value(b);
+	etype v, vdraw = draw_value(b);
 
+#if USE_EGTB
+	if (pop_count32(b->material_mask) < 5 || 
+	    (pop_count32(b->material_mask) == 5 && 
+	     ((b->w_material == KING_VALUE + ROOK_VALUE + KNIGHT_VALUE &&
+	      b->b_material == KING_VALUE + ROOK_VALUE) || 
+	      (b->b_material == KING_VALUE + ROOK_VALUE + KNIGHT_VALUE &&
+	       b->w_material == KING_VALUE + ROOK_VALUE) ||
+	      (b->w_material == KING_VALUE + ROOK_VALUE + BISHOP_VALUE &&
+	       b->b_material == KING_VALUE + ROOK_VALUE) || 
+	      (b->b_material == KING_VALUE + ROOK_VALUE + BISHOP_VALUE &&
+	       b->w_material == KING_VALUE + ROOK_VALUE)))) {
+		
+		EGInit();
+		if (EGTBScore(b, &v)) {
+			if (v == 0)
+				ret = vdraw;
+			else 
+				ret = next_to_play(b)*v;
+			if (debug)
+				lprintf(0, "EGTB: %d\n", ret);
+			b->flags |= FLAG_EGTB_EVAL;
+			return ret - ret0;
+		}
+	}
+#endif
+			
 	if (b->material_mask == b->piece_mask && b->tactics == 0) {
 		/* queen and king vs queen and king is probably a draw */
 		if (b->w_material == KING_VALUE+QUEEN_VALUE && 
@@ -1666,12 +1764,14 @@ static etype check_material(Position *b, etype ret0)
 	if (b->w_material == KING_VALUE+ROOK_VALUE && 
 	    b->b_material <= KING_VALUE+ROOK_VALUE &&
 	    (b->piece_mask & BMINOR_MASK)) {
-		return KNIGHT_VALUE - ROOK_VALUE - NEAR_DRAW_VALUE;
+		ret = -NEAR_DRAW_VALUE;
+		return ret - ret0;
 	}
 	if (b->b_material == KING_VALUE+ROOK_VALUE && 
 	    b->w_material <= KING_VALUE+ROOK_VALUE &&
 	    (b->piece_mask & WMINOR_MASK)) {
-		return ROOK_VALUE - KNIGHT_VALUE + NEAR_DRAW_VALUE;
+		ret = NEAR_DRAW_VALUE;
+		return ret - ret0;
 	}
 
 	/* rook and minor vs rook and pawns is unlikely to be good 
@@ -1680,14 +1780,16 @@ static etype check_material(Position *b, etype ret0)
 	     b->w_material == KING_VALUE+ROOK_VALUE+KNIGHT_VALUE) && 
 	    b->b_material <=  KING_VALUE+ROOK_VALUE+BISHOP_VALUE &&
 	    (b->piece_mask & BROOK_MASK)) {
-		return b->b_material - b->w_material - NEAR_DRAW_VALUE;
+		ret = -NEAR_DRAW_VALUE;
+		return ret - ret0;
 	}
 
 	if ((b->b_material == KING_VALUE+ROOK_VALUE+BISHOP_VALUE || 
 	     b->b_material == KING_VALUE+ROOK_VALUE+KNIGHT_VALUE) && 
 	    b->w_material <=  KING_VALUE+ROOK_VALUE+BISHOP_VALUE &&
 	    (b->piece_mask & WROOK_MASK)) {
-		return b->b_material - b->w_material + NEAR_DRAW_VALUE;
+		ret = NEAR_DRAW_VALUE;
+		return ret - ret0;
 	}
 	
 	if ((b->material_mask & WHITE_MASK) == (b->piece_mask & WHITE_MASK) && 
@@ -1718,7 +1820,7 @@ static etype check_material(Position *b, etype ret0)
 		ret = ((128 - (b->fifty_count-20)) * ret) / 128;
 	}
 
-	if (b->w_material >= b->b_material + (0.8*PAWN_VALUE)) {
+	if (b->w_material >= b->b_material + (0.8*STATIC_PAWN_VALUE)) {
 		/* encourage piece trading but not pawn trading */
 		num = pop_count32(b->piece_mask);
 		ret += piece_trade_bonus[num];
@@ -1726,7 +1828,7 @@ static etype check_material(Position *b, etype ret0)
 		ret -= pawn_trade_bonus[num];
 	}
 
-	if (b->b_material >= b->w_material + (0.8*PAWN_VALUE)) {
+	if (b->b_material >= b->w_material + (0.8*STATIC_PAWN_VALUE)) {
 		/* encourage pawn trading but not piece trading */
 		num = pop_count32(b->piece_mask);
 		ret -= piece_trade_bonus[num];
@@ -1946,16 +2048,22 @@ static etype overloaded_pieces(Position *b)
 
 etype draw_value(Position *b)
 {
-	etype ret = 0;
+	etype ret = DRAW_VALUE;
 
+#if (LEARN_EVAL && !NO_STALEMATE_LEARN)
 	if (state->computer > 0)
-		ret += 0.05*PAWN_VALUE*state->rating_change;
+		return ret;
+	else 
+		return -ret;
+#endif
+	if (state->computer > 0)
+		ret += 0.05*STATIC_PAWN_VALUE*state->rating_change;
 	else
-		ret -= 0.05*PAWN_VALUE*state->rating_change;
-	
-	if (ABS(ret) > PAWN_VALUE)
-		ret = SIGN(ret)*PAWN_VALUE;
+		ret -= 0.05*STATIC_PAWN_VALUE*state->rating_change;
 	
+	if (ABS(ret) > STATIC_PAWN_VALUE)
+		ret = SIGN(ret)*STATIC_PAWN_VALUE;
+
 	return ret;
 }
 
@@ -2105,15 +2213,6 @@ etype eval_etype(Position *b, etype testv, int depth)
 		}
 	}
 
-#if 0
-	if (b->hash1 == 0x231672de) {
-		debug = 1;
-		print_board(b->board);
-	} else {
-		debug = 0;
-	}
-#endif
-
 	init_eval_tables();
 
 	/* passed pawns always get reevaluated, so stopppability does also */
@@ -2305,7 +2404,7 @@ etype eval_etype(Position *b, etype testv, int depth)
 	    b->oldb &&
 	    abs(b->expensive) < EVAL_SHORTCUT_THRESHOLD &&
 	    (ret + material + b->expensive) * player > 
-	    testv + EVAL_SHORTCUT_THRESHOLD) {
+	    testv + EVAL_SHORTCUT_THRESHOLD && !egtb(b)) {
 		etype ret2 = (ret + material + b->expensive) * player;
 		
 		ret2 -= EVAL_SHORTCUT_OFFSET;
@@ -2402,10 +2501,12 @@ Eval eval(Position *b, etype testv, int depth)
 {
 	Eval ret = makeeval(b, eval_etype(b, testv, depth));
 
-	if (ret.v > MAX_EVAL)
-		ret.v = MAX_EVAL;
-	if (ret.v < -MAX_EVAL)
-		ret.v = -MAX_EVAL;
+	if (!(b->flags & FLAG_EGTB_EVAL)) {
+		if (ret.v > MAX_EVAL)
+			ret.v = MAX_EVAL;
+		if (ret.v < -MAX_EVAL)
+			ret.v = -MAX_EVAL;
+	}
 	return ret;
 }
 
