@@ -25,7 +25,7 @@ static int opponent;
 static int always_think;
 static int no_display;
 static int ascii_display;
-static int use_mulling=1;
+static int use_mulling = USE_PBRAIN;
 static int display_position=0;
 static int brain_inserting=0;
 static int ics_mode;
@@ -56,6 +56,7 @@ char *ics_name = "KnightCap";
 char *ics_master = NULL;
 char *coeffs_file = NULL;
 char *bad_eval_file = "bad_eval.dat";
+char *tb_path = "../EGTB";
 
 char play_target[100];
 
@@ -67,7 +68,7 @@ char play_target[100];
 #define SHM_W 0200
 #endif
 
-void sig_wake_up(void)
+void sig_wake_up(int dum)
 {
 	signal(SIGUSR1, sig_wake_up);
 }
@@ -91,11 +92,15 @@ void reset_board(void)
 		lprintf(0,"Thinking on opponents time\n");
 	timer_reset();
 	state->analysed = 0;
+	state->first_book_move = 1;
+	estimate_game_stage(&state->position, 1);
 #if LEARN_EVAL
-	state->stored_move_num = 0;
 	learning=1;
 	state->won = UNKNOWN;
-	td_init();
+#endif
+#if STORE_LEAF_POS
+	state->stored_move_num = 0;
+	memset(state->predicted_move, 0, MAX_GAME_MOVES*sizeof(state->predicted_move[0]));
 #endif
 }
 
@@ -225,12 +230,12 @@ int parse_stdin(char *line)
 		}
 
 		if (sscanf(tok, "time %d", &hsecs) == 1) {
-			timer_estimate(hsecs/100, increment);
+			timer_estimate(hsecs/100, hsecs/100, increment);
 			done = 1;
 		}
 
 		if (sscanf(tok, "mtim %d %d", &hsecs, &increment) == 2) {
-			timer_estimate((60*hsecs), increment);
+			timer_estimate((60*hsecs), (60*hsecs), increment);
 			done = 1;
 		}
 
@@ -385,11 +390,14 @@ int parse_stdin(char *line)
 			done = 1;
 		}
 
-		if (sscanf(tok,"grad %d", &i) == 1) {
+		if (strcmp(tok,"grad") == 0) {
 #if LEARN_EVAL
-			state->position.stage = i;
-			state->stored_move_num = 1;
-			td_gradient(&state->position);
+			float grad[__TOTAL_COEFFS__];
+
+			state->stored_move_num = 0;
+			td_store_pos(&state->position);
+			state->td_comp = next_to_play(&state->position);
+			td_gradient(grad);
 #endif
 			done = 1;
 		}
@@ -658,10 +666,13 @@ void idle_func(void)
 
 	}
 
-	if (state->auto_exit && state->moves_mulled >= MAX_MOVES_MULLED) {
+	if (state->auto_exit && state->moves_mulled >= MAX_MOVES_MULLED && state->open) {
 		lprintf(0,"***%d\n", state->moves_mulled);
 		state->use_mulling=0;
 		state->stop_search=1;
+		sleep(5);
+		system("killall sleep");
+		sleep(5);
 		system("killall sleep");
 		exit(0);
 	}
@@ -808,7 +819,7 @@ static void move_thread(void)
 
 	while (!state->quit) {
 		Move move;
-
+		
 		if (!process_exists(display_pid)) {
 #if USE_PBRAIN
 			brain_close();
@@ -834,7 +845,7 @@ static void move_thread(void)
 #endif			
 
 
-		if (state->computer == 0 && state->use_mulling) {
+		if (state->computer == 0 && !state->analysed) {
 			/* if we get bored waiting then start to mull on
 			   our brain */
 			analyse_game();
@@ -868,7 +879,7 @@ static void move_thread(void)
 				}
 			}
 			
-			if (!pondered)
+			if (!pondered || !state->computer)
 				usleep(SLEEP_TIME);
 			continue;
 		}
@@ -927,7 +938,7 @@ static void create_threads(void)
 	shmctl(shmid, IPC_RMID, 0);
 	memset(state, 0, len);
 	state->move_time = default_move_time;
-	state->use_mulling = 1;
+	state->use_mulling = USE_PBRAIN;
 	state->use_pbrain = 1;
 	state->hash_table_size = def_hash_table_size;
 
@@ -985,6 +996,9 @@ static void usage(void)
 "-A                  always think\n" \
 "-H <size>           set hash table size in MB\n" \
 "-f <file>           get eval coefficients from this file\n" \
+"-g <file>           EGTB directory\n" \
+"-T <name>           target ICS opponent\n" \
+"-G <file>           endgame tablebsae directory\n" \
 "-y                  don't mull\n" \
 "\n");
 }
@@ -996,7 +1010,7 @@ static void parse_options(int argc,char *argv[])
 #ifdef SUNOS4
 	extern char *optarg;
 #endif
-	char *opt = "c:s:w:nI:t:hXAD:l:aH:M:ezdm:f:ByEp";
+	char *opt = "c:s:w:nI:t:hXAD:l:aH:M:ezdm:f:g:ByEpT:";
 	int c;
 	int seed = getpid();
 
@@ -1059,16 +1073,18 @@ static void parse_options(int argc,char *argv[])
 
 		case 'z':
 			seek_answering = 1;
-			f = fopen("opponents.dat", "r");
-			if (f) {
+			f = (FILE *)fopen("opponents.dat", "r");
+			if (f != NULL) {
 				fscanf(f, "%s", opponents_list);
 			}
-			fclose(f);
-			f = fopen("noplay.dat", "r");
-			if (f) {
+			if (f != NULL)
+				fclose(f);
+			f = (FILE *)fopen("noplay.dat", "r");
+			if (f != NULL) {
 				fscanf(f,"%s", noplay_list);
 			}	
-			fclose(f);
+			if (f != NULL)
+				fclose(f);
 			lprintf(0, "answering computer seeks %s %s\n",
 				opponents_list, noplay_list);
 			break;
@@ -1129,6 +1145,12 @@ static void parse_options(int argc,char *argv[])
 		case 'p':
 			display_position = 1;
 			break;     
+	        case 'T':
+			strcpy(play_target, optarg);
+			break;
+		case 'g':
+			tb_path = optarg;
+			break;
 		}
 	}
 
@@ -1153,12 +1175,6 @@ int main(int argc,char *argv[])
 
 	lprintf(0,"KnightCap starting\ntype \"help\" for commands or use the right button\n");
 
-#if LEARN_EVAL
-	if ((bad_eval_fd = open(bad_eval_file, O_RDWR)) == -1) {
-		lprintf(0,"Failed to open %s\n", bad_eval_file);
-	}
-#endif
-
 	parse_options(argc, argv);
 
 	create_threads();
@@ -1171,10 +1187,11 @@ int main(int argc,char *argv[])
 	state->use_mulling = use_mulling;
 	state->brain_inserting = brain_inserting;
 	state->open = 1;
-
+	state->mulling = 0;
+	state->notified = 0;
 	reset_board();
 	init_eval_tables();
-	sig_wake_up();
+	sig_wake_up(0);
 
 #if RENDERED_DISPLAY
 	if (!no_display && !ascii_display) {
