@@ -1,10 +1,15 @@
 #if LARGE_ETYPE
 typedef int etype;
+#if USE_STATIC_PAWN_VALUE 
+#define STATIC_PAWN_VALUE 1000
+#else 
+#define STATIC_PAWN_VALUE 10000
 #define PAWN_VALUE 10000
+#endif
 #else
 typedef short etype;
+#define STATIC_PAWN_VALUE 100
 #define PAWN_VALUE 100
-#define LARGE_ETYPE 0
 #endif
 
 #include "eval.h"
@@ -22,19 +27,26 @@ typedef short etype;
 #define SHORT_GAME 10
 #endif
 
+#define STORE_ONLY_NEGATIVE 0
+#define USE_APPLET 0
+#define SEARCH_OPENING_VARIATIONS 0
+
 #define DUMPING_TD_UPDATES 1
 
-#define NO_STALEMATE_LEARN 1 
+#define NO_STALEMATE_LEARN 1
 
+#define SEARCH_DEBUG 0
 #define __COEFFS_PER_STAGE__ (__TOTAL_COEFFS__/(MATING+1))
 #define USE_HASH_TABLES 1
 #define USE_ETTC_HASH 1
-#define USE_PAWN_DEEPENING 1
+#define USE_PAWN_DEEPENING 0
 #define USE_CHECK_DEEPENING 1
 #define USE_RECAPTURE_DEEPENING 1
 #define USE_BEST_CAPTURE_DEEPENING 0
 #define USE_HUNG_CAPTURE_DEEPENING 0
 #define USE_SINGULAR_EXTENSION 1
+#define USE_EVAL_TACTICS 1
+#define USE_EGTB 1
 
 #define USE_NULL_MOVE 1
 #define USE_NULLMOVE_FIXUP 1
@@ -44,11 +56,11 @@ typedef short etype;
 #define USE_RAZORING 1
 #define USE_ASYMMETRIC_RAZORING 0
 #define USE_EVAL_SHORTCUT 1
-#define USE_QUIESCE_SHORTCUT 1
+#define USE_QUIESCE_SHORTCUT 0
 #define USE_PESSIMISTIC_PRUNING 0
 #define USE_ASYMMETRIC_EXTENSIONS 1
 #define USE_ASYMMETRIC_EVAL 1
-#define USE_ASYMMETRIC_NULLMOVE 0
+#define USE_ASYMMETRIC_NULLMOVE 1
 
 #ifndef USE_SLIDING_CONTROL
 #define USE_SLIDING_CONTROL 1
@@ -56,6 +68,9 @@ typedef short etype;
 
 #define USE_FUTILITY 1
 #define USE_FAST_QUIESCE 0
+#define USE_PAWNPUSH_QUIESCE 0
+
+#define REJECT_LOW_BRAIN_MOVES 0
 
 #define TEST_QUIESCE 0
 #define TEST_EVAL_SHORTCUT 0
@@ -64,7 +79,7 @@ typedef short etype;
 #define TEST_MTD_FAST 0
 #define SAVE_EXTENSIONS 0
 
-#define FUTILE_THRESHOLD (1.3*PAWN_VALUE)
+#define FUTILE_THRESHOLD (1.3*STATIC_PAWN_VALUE)
 
 #define USE_INTERNAL_PRUNING 0
 #define USE_HALF_PRUNING 0
@@ -77,12 +92,10 @@ typedef short etype;
 #define USE_PBRAIN 1
 #endif
 
-#define CONJUNCTION_CALC 0
-
 #define BRAIN_FILE "brain.dat"
 
 #ifndef RESIGN_VALUE
-#define RESIGN_VALUE (INFINITY)
+#define RESIGN_VALUE (ROOK_VALUE)
 #endif 
 
 #define NULL_MOVE_PLY 3
@@ -90,14 +103,14 @@ typedef short etype;
 
 #define MAX_EXTENSIONS 20
 
-#define RAZOR_THRESHOLD (0.9*PAWN_VALUE)
+#define RAZOR_THRESHOLD (0.9*STATIC_PAWN_VALUE)
 
 #define MAX_MOVES_MULLED 3
 
 #define NULL_MOVE_THRESHOLD 6
 
-#define MTD_THRESHOLD (0.07*PAWN_VALUE)
-#define MTD_DELTA (0.05*PAWN_VALUE)
+#define MTD_THRESHOLD (mulling?(0.01*STATIC_PAWN_VALUE):(0.07*STATIC_PAWN_VALUE))
+#define MTD_DELTA (0.05*STATIC_PAWN_VALUE)
 
 #define MAX_CELLS 64
 
@@ -106,7 +119,7 @@ typedef short etype;
 #define CAPTURE_SEARCH_DEPTH 0
 
 #define USE_FORWARD_PRUNING 0
-#define PRUNE_THRESHOLD (PAWN_VALUE/4)
+#define PRUNE_THRESHOLD (STATIC_PAWN_VALUE/4)
 
 #define PAR_SPLIT_LEVEL 1
 
@@ -203,12 +216,12 @@ typedef short etype;
 #define BMAJOR_MASK (BPIECE_MASK & ~BMINOR_MASK)
 #define MAJOR_MASK  (WMAJOR_MASK | BMAJOR_MASK)
 
-#define INFINITY (40.50*PAWN_VALUE)
-#define ILLEGAL (-40*PAWN_VALUE)
+#define INFINITY (40.50*STATIC_PAWN_VALUE)
+#define ILLEGAL (-40*STATIC_PAWN_VALUE)
 #define WIN (-(ILLEGAL+1))
-#define FORCED_WIN (WIN-50)
+#define FORCED_WIN (WIN-127)
 #define MAX_EVAL (FORCED_WIN-20)
-#define MAX_MATERIAL (30.00*PAWN_VALUE)
+#define MAX_MATERIAL (30.00*STATIC_PAWN_VALUE)
 
 #define STALEMATE 2
 #define TIME_FORFEIT 3
@@ -254,7 +267,7 @@ typedef short etype;
 #define FLAG_NEED_PART2       (1<<20)
 #define FLAG_FORCED_DRAW      (1<<21)
 #define FLAG_BAD_EVAL         (1<<22)
-#define FLAG_BRAIN_EVAL       (1<<23)
+#define FLAG_EGTB_EVAL        (1<<23)
 #define FLAG_GAME_OVER        (1<<24)
 #define FLAG_LOST             (1<<25)
 #define FLAG_EXTENDED         (1<<26)
@@ -272,6 +285,9 @@ typedef short etype;
 
 #define mat_value(x) new_coefficients[IPIECE_VALUES + (x)]
 
+#if USE_STATIC_PAWN_VALUE
+#define PAWN_VALUE mat_value(PAWN)
+#endif
 #define KNIGHT_VALUE mat_value(KNIGHT)
 #define BISHOP_VALUE mat_value(BISHOP)
 #define ROOK_VALUE mat_value(ROOK)
@@ -317,7 +333,6 @@ typedef struct {
 	int v;
 } Move;
 
-#if STORE_LEAF_POS
 typedef struct {
 	Piece board[NUM_SQUARES];
 	uint32 flags;
@@ -326,7 +341,6 @@ typedef struct {
 	Square enpassent;
 	unsigned char fifty_count;
 } SmallPos;
-#endif
 
 typedef struct {
 	/* use a short to hold evaluations */
@@ -363,6 +377,7 @@ struct hash_entry {
 	unsigned from:6;
 	unsigned to:6;
 	unsigned tag:4;
+	unsigned char move_num;
 };
 
 typedef struct Position {
@@ -483,14 +498,28 @@ struct state {
 	char black_player[100];
 	int moves_mulled;
 	int analysed;
+	int krating;
+	int orating;
+	int wtime;
+	int btime;
+	int game_time;
+	int increment;
+	int mulling;
+	int current_depth;
+	int last_book_move;
+	int notified;
+	int td_comp;
+	int converged;
+	int first_book_move;
+	int first_book_move_index;
+	int total_rounds;
+	char notifyee[256];
+	char game_type[20];
 #if STORE_LEAF_POS
 	LeafEval leaf_eval[MAX_GAME_MOVES];
-	etype grad[__TOTAL_COEFFS__];
+	Position leaf_pos[MAX_GAME_MOVES];
 	short predicted_move[MAX_GAME_MOVES];
 	int stored_move_num;
-	int buffered_moves;
-	int gradient_fd;
-	etype grad_buffer[GRAD_TEMP_SIZE];
 #endif
 };
 
@@ -500,6 +529,7 @@ struct state {
 
 
 #include "proto.h"
+#include "epdglue.h"
 
 #ifdef i386
 static inline int ff_one(uint32 word)
