@@ -4,14 +4,17 @@
 static int pondered_wrong_move;
 static int nodes, quiesce_nodes, extended_nodes, truncated_nodes, confirm_nodes;
 static int maxply, maxdepth;
-static int move_order_hits, move_order_misses;
+static int move_order_hits, move_order_hits1, move_order_hits2, 
+	move_order_hits3, move_order_hits4, move_order_misses;
 static int pondering, ponder_stop;
 static int search_depth;
+
 extern int mulling;
+extern int mull_stage;
 extern int demo_mode;
-extern int bad_eval_fd;
 extern int learning;
 extern int pop_count[256];
+extern int hash_hits, hash_misses, hash_shallow;
 
 static volatile Position *real_position;
 static volatile Position *ponder_position;
@@ -22,6 +25,8 @@ static int tree_print_ply = TREE_PRINT_PLY;
 
 extern struct state *state;
 
+extern int testing;
+
 static int null_ply[MAX_DEPTH+10];
 
 
@@ -34,10 +39,12 @@ static int search_expired(void)
 		if (state->computer) {
 			return 1;
 		}
-		return 0;
 	} else {
 		if (state->computer == 0)
-			return 1;
+			if (testing)
+				return 0;
+			else 
+				return 1;
 	}
 	
 	if (pondering) {
@@ -48,14 +55,15 @@ static int search_expired(void)
 			return 0;
 		if (real_position->hash1 != ponder_position->hash1) {
 #if STORE_LEAF_POS
-			state->predicted_move[state->stored_move_num] = 0;
+			state->predicted_move[state->stored_move_num] = -1;
 #endif
 			pondered_wrong_move = 1;
 			return 1;
 		}
 		lprintf(0,"predicted move\n");
 #if STORE_LEAF_POS
-		state->predicted_move[state->stored_move_num] = 1;
+		if (state->predicted_move[state->stored_move_num] == 0)
+			state->predicted_move[state->stored_move_num] = 1;
 #endif
 		
 		pondering = 0;
@@ -65,7 +73,7 @@ static int search_expired(void)
 	/* check for timer expiry every few nodes (too expensive to
 	   check every node, and alarm isn't accurate enough for 
 	   lightning) */
-	if (search_depth > 2) {
+	if ((search_depth > 2)  && (!mulling || (search_depth >= state->current_depth + 1 && state->converged))) {
 		static int last_test;
 		if (nodes < last_test) {
 			last_test = nodes;
@@ -92,11 +100,13 @@ static inline int depth_extensions(Position *b, int depth, int ply,
 
 #if USE_PAWN_DEEPENING
 	if ((b->board[b->last_move.to] == PAWN && 
+	     (b->wpassed_pawn_mask & (1<<b->pboard[b->last_move.to])) && 
 	     YPOS(b->last_move.to) > 4 && 
 	     b->control[b->last_move.to] >= 0 && 
 	     b->board[b->last_move.to+NORTH] == 0 &&
 	     b->control[b->last_move.to+NORTH] >= 0) ||
 	    (b->board[b->last_move.to] == -PAWN && 
+	     (b->bpassed_pawn_mask & (1<<b->pboard[b->last_move.to])) && 
 	     YPOS(b->last_move.to) < 3 &&
 	     b->control[b->last_move.to] <= 0 && 
 	     b->board[b->last_move.to+SOUTH] == 0 &&
@@ -129,7 +139,7 @@ static inline int depth_extensions(Position *b, int depth, int ply,
 			int mat1 = b->w_material - b->b_material;
 			int mat2 = b->oldb->oldb->w_material - 
 				b->oldb->oldb->b_material;
-			if (abs(mat1 - mat2) < PAWN_VALUE/2) {
+			if (abs(mat1 - mat2) < ABS(PAWN_VALUE)/2) {
 				b->flags |= FLAG_EXTENDED;		
 				return depth+1;
 			}
@@ -450,6 +460,35 @@ static Eval quiesce(Position *b, Eval testv, int ply)
 			}
 		}
 
+#if USE_PAWNPUSH_QUIESCE
+		/* push any passed pawns beyond the third rank */
+		if (b->wpassed_pawn_mask) {
+			pieces = b->wpassed_pawn_mask;
+			while (pieces) {
+				i = ff_one(pieces);
+				pieces &= ~(1<<i);
+				if (YPOS(b->pieces[i].pos) < 3 || YPOS(b->pieces[i].pos) > 5) 
+					continue;
+				if (b->board[b->pieces[i].pos+NORTH]) 
+					continue;
+
+				m1.from = b->pieces[i].pos;
+				m1.to = b->pieces[i].pos + NORTH;
+
+				if (!do_move_part1(&b1, b, &m1)) continue;
+
+				v = quiesce(&b1, flip(testv), ply+1);
+				
+				v.v = -v.v;
+				if (v.v > g.v) {
+					g = v;
+					if (g.v >= testv.v) 
+						goto evaluated;
+				}
+			}
+		}
+#endif 
+			
 		/* try the "best capture" move from the tactical eval */
 		m1 = b->best_capture;
 		if (!is_zero_move(&m1) && b->board[m1.to] < 0 &&
@@ -576,6 +615,35 @@ static Eval quiesce(Position *b, Eval testv, int ply)
 			}
 		}
 
+#if USE_PAWNPUSH_QUIESCE
+		/* push any passed pawns beyond the third rank */
+		if (b->bpassed_pawn_mask) {
+			pieces = b->bpassed_pawn_mask;
+			while (pieces) {
+				i = ff_one(pieces);
+				pieces &= ~(1<<i);
+				if (YPOS(b->pieces[i].pos) > 4 || YPOS(b->pieces[i].pos) < 2) 
+					continue;
+				if (b->board[b->pieces[i].pos+SOUTH]) 
+					continue;
+
+				m1.from = b->pieces[i].pos;
+				m1.to = b->pieces[i].pos + SOUTH;
+
+				if (!do_move_part1(&b1, b, &m1)) continue;
+
+				v = quiesce(&b1, flip(testv), ply+1);
+				
+				v.v = -v.v;
+				if (v.v > g.v) {
+					g = v;
+					if (g.v >= testv.v) 
+						goto evaluated;
+				}
+			}
+		}
+#endif
+
 		/* try the "best capture" move from the tactical eval */
 		m1 = b->best_capture;
 		if (!is_zero_move(&m1) && b->board[m1.to] > 0 &&
@@ -923,7 +991,7 @@ static Eval abtest(Position *b, int depth0, Eval testv, int ply)
 
 		/* don't let it see false zugzwang mates */
 		if (v.v >= WIN) {
-			v.v -= PAWN_VALUE;
+			v.v -= STATIC_PAWN_VALUE;
 		}
 		
 		if (v.v >= testv.v) {
@@ -945,8 +1013,8 @@ static Eval abtest(Position *b, int depth0, Eval testv, int ply)
 #if USE_PESSIMISTIC_PRUNING
 	if (depth <= 3 && computers_move(b)) {
 		v = quiesce(b, testv, ply+1);
-		if (v.v < testv.v - PAWN_VALUE/2) {
-			v.v += PAWN_VALUE/4;
+		if (v.v < testv.v - STATIC_PAWN_VALUE/2) {
+			v.v += STATIC_PAWN_VALUE/4;
 			g = v;
 			goto evaluated;
 		}
@@ -1005,10 +1073,10 @@ static Eval abtest(Position *b, int depth0, Eval testv, int ply)
 
 #if 0
 		/* a crazy selective search idea */
-		if (m > 0 && g.v > testv.v - PAWN_VALUE/2) {
+		if (m > 0 && g.v > testv.v - ABS(PAWN_VALUE)/2) {
 			v2.v = eval_move_tactics(b, &moves[m]);
 			if (v2.v < 0 && 
-			    v2.v + v1.v < testv.v - 0.8*PAWN_VALUE) {
+			    v2.v + v1.v < testv.v - 0.8*ABS(PAWN_VALUE)) {
 				continue;
 			}
 		}
@@ -1083,6 +1151,14 @@ static Eval abtest(Position *b, int depth0, Eval testv, int ply)
 			} else {
 				move_order_misses++;
 			}
+			if (m==1) 
+				move_order_hits1++;
+			else if (m==2)
+				move_order_hits2++;
+			else if (m==3) 
+				move_order_hits3++;
+			else if (m!=0)
+				move_order_hits4++;
 			cutoff_hint(b, m, depth, ply);
 			goto end_loop;
 		}
@@ -1547,8 +1623,9 @@ static Eval mtd_search(Position *b, Move *move, int depth,
 	int loops=0;
 	int alpha_count=0;
 	int beta_count=0;
+#if SEARCH_DEBUG	
 	struct hash_entry *h;
-
+#endif
 	alpha = makeeval(b, ILLEGAL);
 	beta = makeeval(b, -ILLEGAL);
 	g = guess;
@@ -1559,7 +1636,6 @@ static Eval mtd_search(Position *b, Move *move, int depth,
 
 	/* generate the move list */
 	root_moves(b, move);
-
 	/* try to get a lower bound from the hash table */
 #if USE_ETTC_HASH
 	if (ettc_check_hash(b, b->moves, b->num_moves,
@@ -1572,13 +1648,13 @@ static Eval mtd_search(Position *b, Move *move, int depth,
 		static float offsets[7] = {
 			0.5, 2, 6, 13, 30, 30, 30};
 		if (loops > 6) break;
-		y.v = g.v - offsets[loops]*PAWN_VALUE;
+		y.v = g.v - offsets[loops]*STATIC_PAWN_VALUE;
 
 		if (y.v <= alpha.v)
-			y.v = alpha.v + 0.01*PAWN_VALUE;
+			y.v = alpha.v + 0.01*STATIC_PAWN_VALUE;
 
 		if (y.v >= beta.v)
-			y.v = beta.v - 0.01*PAWN_VALUE;
+			y.v = beta.v - 0.01*STATIC_PAWN_VALUE;
 
 		g = abroot(b, depth, y, move);
 
@@ -1590,8 +1666,8 @@ static Eval mtd_search(Position *b, Move *move, int depth,
 				return alpha;
 			return guess;
 		}
-
-#if 0
+	
+#if SEARCH_DEBUG
 		lprintf(0,"mtd g=%d y=%d alpha=%d beta=%d nodes=%d %s\n",
 			g.v, y.v, alpha.v, beta.v, nodes,
 			short_movestr(b, move));
@@ -1640,20 +1716,9 @@ static Eval mtd_search(Position *b, Move *move, int depth,
 		if (alpha.v > ILLEGAL && beta.v < -ILLEGAL &&  !mulling &&
 		    mtd_converged(b, alpha.v, beta.v, depth, &y))
 			break;
-#if 0
-		if (y.v <= alpha.v)
-			y.v = alpha.v + 1;
 
-		if (y.v > beta.v)
-			y.v = beta.v;
-#endif
 		g = abroot(b, depth, y, move);
 
-#if 0
-		lprintf(0,"mtd g=%d y=%d alpha=%d beta=%d nodes=%d %s\n",
-			g.v, y.v, alpha.v, beta.v, nodes,
-			short_movestr(b, move));
-#endif
 		mtd_count++;
 
 		if (search_expired()) {
@@ -1668,7 +1733,7 @@ static Eval mtd_search(Position *b, Move *move, int depth,
 			alpha = g;
 		}
 
-#if 0
+#if SEARCH_DEBUG
 		lprintf(0,"mtd g=%d y=%d alpha=%d beta=%d nodes=%d %s\n",
 			g.v, y.v, alpha.v, beta.v, nodes,
 			short_movestr(b, move));
@@ -1685,13 +1750,11 @@ static Eval mtd_search(Position *b, Move *move, int depth,
 			break;
 		}
 
-#if 1
 		if (alpha.v >= beta.v - MTD_THRESHOLD &&
 		    (mtd_count > 30 || 
 		     (abs(alpha.v) < FORCED_WIN && !mulling))) {
 			break;
 		}
-#endif
 
 	}
 
@@ -1707,6 +1770,13 @@ static Eval mtd_search(Position *b, Move *move, int depth,
 static Eval search_one_depth(Position *b, Move *move, 
 			     int depth, Eval guess)
 {
+	move_order_misses = 0;
+	move_order_hits = 0;
+	move_order_hits1 = 0;
+	move_order_hits2 = 0;
+	move_order_hits3 = 0;
+	move_order_hits4 = 0;
+
 #if APLINUX
 	return par_search(b, move, depth, guess);
 #else
@@ -1738,8 +1808,8 @@ static void show_pv(Position *b, Move *move, int exporting)
 		if (exporting) {
 			status_printf(0, "%s(%4.2lf:%4.2lf) ", 
 				      short_movestr(&b1, &m1),
-				      (double)(t->low.v)/PAWN_VALUE, 
-				      (double)(t->high.v)/PAWN_VALUE);
+				      (double)(t->low.v)/STATIC_PAWN_VALUE, 
+				      (double)(t->high.v)/STATIC_PAWN_VALUE);
 		} else {
 			lprintf(0,"%s(%d:%d) ", 
 				short_movestr(&b1, &m1),
@@ -1771,24 +1841,27 @@ static void show_eval(Position *b, Eval v, int edepth)
 		return;
 	}
 
-	/* MATING stage is pretty irrelevant at the moment */
-	if (v.pos.stage == MATING) {
-		lprintf(0, "mating stage\n");
+	/* occasionally we get a futility node here */
+	if (v.pos.flags & FLAG_FUTILE) {
+		lprintf(0,"****futility node\n");
 		return;
 	}
 
-	/* occasionally we get a futility node here */
-	if (v.pos.flags & FLAG_FUTILE || v.pos.flags & FLAG_BRAIN_EVAL) {
+	/* EGTB nodes are no good */
+	if (v.pos.flags & FLAG_EGTB_EVAL) {
+		lprintf(0,"****EGTB node\n");
 		return;
 	}
 
 	/* all features are ignored for forced draws  */
 	if ((v.pos.flags & FLAG_FORCED_DRAW) || (v.pos.flags & FLAG_ACCEPT_DRAW)) {
+		lprintf(0,"****forced draw\n");
 		return;
 	}
 
 	/* leaf position is irrelevant for forced mate */
 	if  (v.v < -FORCED_WIN || v.v > FORCED_WIN) {
+		lprintf(0,"****forced win %d\n", v.v);
 		return;
 	}
 		
@@ -1819,12 +1892,13 @@ static void show_eval(Position *b, Eval v, int edepth)
 	f2 = tanh(v1.v*EVAL_SCALE*next_to_play(&b1));
 
 	if (ABS(f1-f2) < EVAL_DIFF) {
-		td_gradient(&b1);
+		td_store_pos(&b1);
 	} else {
 		lprintf(0,"***EVAL DIFF***\n %d %d\n", 
 			v.v*next_to_play(b), v1.v*next_to_play(&b1));
 		print_board(v.pos.board);
 		eval_debug(&b1);
+		hash_reset();
 	}
 #endif
 }
@@ -1842,7 +1916,7 @@ static int terrible_move(Position *b, Move *move, Eval *v, int depth)
 	m1.from = t->from;
 	m1.to = t->to;
 
-	if (t->high.v > PAWN_VALUE-v->v) {
+	if (t->high.v > ABS(PAWN_VALUE)-v->v) {
 		lprintf(0,"unbounded: %s %d low=%d high=%d %s\n",
 			short_movestr(b, move), v->v, t->low.v, t->high.v,
 			short_movestr(&b1, &m1));
@@ -1860,6 +1934,7 @@ static Eval search(Position *b, Move *move)
 	Eval v, lastv;
 	int edepth;
 	int depth;
+	int tdepth;
 	int terminate=0;
 	Eval guess, v1, lastv1;
 	int expired, force;
@@ -1872,7 +1947,6 @@ static Eval search(Position *b, Move *move)
 	confirm_nodes = 0;
 	extended_nodes = 0;
 	truncated_nodes = 0;
-	move_order_misses = move_order_hits = 0;
 	hash_reset_stats();
 
 	v.v = lastv.v = 0;
@@ -1883,7 +1957,7 @@ static Eval search(Position *b, Move *move)
 	depth = 2;
 
 	t = fetch_hash(b);
-	if (t) {
+	if (t && !t->move_num) {
 		move->from = t->from;
 		move->to = t->to;
 		if (!is_zero_move(move)) {
@@ -1898,6 +1972,8 @@ static Eval search(Position *b, Move *move)
 	}
 
 	edepth = depth-1;
+	
+	zero_move(&last_move);
 
 	while (!terminate && depth < MAX_DEPTH) {
 
@@ -1909,14 +1985,13 @@ static Eval search(Position *b, Move *move)
 
 		guess = v;
 		lastv = v;
-		last_move = *move;
+
 
 		counter = 0;
 
 	again:
 		counter++;
 		v = search_one_depth(b, move, depth, guess);
-		
 
 		expired = timer_expired();
 		if (pondering)
@@ -1939,16 +2014,18 @@ static Eval search(Position *b, Move *move)
 		if (pondering || edepth <= 2) {
 			terminate = 0;
 		} else {
-			terminate = timer_terminate(depth, next_to_play(b), force);
+			if (!search_expired()) 
+				edepth = depth;
+			if (!mulling || edepth >= state->current_depth) 
+				terminate = timer_terminate(depth, next_to_play(b), force);
 		}
-
-		if (!search_expired()) {
+		
+		if (!search_expired()) 
 			edepth = depth;
-		} else {
+		else 
 			terminate = 1;
-		}
 
-		lprintf(0, "%s%s: %s d=%d n=%d c=%d%% q=%d%% ext=%d%% trunc=%d%% mo=%d%% %s mtd=%d t=%.1f %s nps=%d\n", 
+		lprintf(0, "%s%s: %s d=%d n=%d c/q/e/t=%d/%d/%d/%d%% mo=%d%% %s mtd=%d t=%.1f %s nps=%d\n", 
 			pondering?"** ":"",
 			colorstr(next_to_play(b)), 
 			evalstr(v.v),
@@ -1965,6 +2042,8 @@ static Eval search(Position *b, Move *move)
 			short_movestr(b, move),
 			(int)((nodes+quiesce_nodes)/timer_elapsed()));
 
+		hash_hits = hash_misses= hash_shallow = 0;
+
 		if (edepth > 0 && depth > 4) {
 			status_printf(1, "%s%s: %s d=%d ",
 				      pondering?"** ":"",
@@ -1984,16 +2063,16 @@ static Eval search(Position *b, Move *move)
 		}
 
 		if (state->ics_robot && edepth > 3 && !mulling) {
-			if (!pondering && v1.v > lastv1.v + 4*PAWN_VALUE && 
-			    lastv1.v < 3*PAWN_VALUE && v1.v > -2*PAWN_VALUE) {
+			if (!pondering && v1.v > lastv1.v + 4*STATIC_PAWN_VALUE && 
+			    lastv1.v < 3*STATIC_PAWN_VALUE && v1.v > -2*STATIC_PAWN_VALUE) {
 				prog_printf("whisper Eval=%s yeah!\n", 
 					    evalstr(v1.v));
-			} else if (!pondering && lastv1.v < 0.9*PAWN_VALUE && v1.v > 1.3*PAWN_VALUE) {
+			} else if (!pondering && lastv1.v < 0.9*STATIC_PAWN_VALUE && v1.v > 1.3*STATIC_PAWN_VALUE) {
 				prog_printf("whisper Eval=%s looking good\n", 
 					    evalstr(v1.v));
-			} else if (v1.v < lastv1.v - 0.9*PAWN_VALUE && lastv1.v < 0) {
+			} else if (v1.v < lastv1.v - 0.9*STATIC_PAWN_VALUE && lastv1.v < 0) {
 				prog_printf("whisper Eval=%s uh oh\n", evalstr(v1.v));
-			} else if (v1.v < lastv1.v - 4*PAWN_VALUE && lastv1.v < 4*PAWN_VALUE) {
+			} else if (v1.v < lastv1.v - 4*STATIC_PAWN_VALUE && lastv1.v < 4*STATIC_PAWN_VALUE) {
 				prog_printf("whisper Eval=%s damn\n", evalstr(v1.v));
 			}
 		}
@@ -2001,35 +2080,49 @@ static Eval search(Position *b, Move *move)
 		if (v.v <= -(WIN-edepth) || v.v >= (WIN-edepth))
 			terminate = 1;
 
-		if (terminate && !pondering && state->ics_robot &&
-		    state->move_time > 2 && !mulling) {
-			prog_printf("whisper Eval=%s nps=%d depth=%d\n",
+		if (terminate && !pondering && state->ics_robot && state->move_time > 0.7 && !mulling) {
+			prog_printf("whisper Eval=%s nps=%d depth=%d cpu=%.1f%%\n",
 				    evalstr(v1.v), 
 				    (int)((nodes+quiesce_nodes)/timer_elapsed()),
-				    edepth);
+				    edepth,
+				    cpu_percent());
 		}
 		
-		if (terminate && !pondering && edepth > 0) {
-			show_pv(b,move,0);
+		if (terminate) {
+			if (!mulling && !pondering) {
+				show_pv(b,move,0);
 #if LEARN_EVAL
-			if (!mulling) {
 				if (v.v > -FORCED_WIN || v.v < FORCED_WIN) {
 					if (small_window) {
 						show_eval(b,v,edepth);
-						state->converged_record[state->position.move_num] = *move; 
-					} else if (lastv.v != 0) {
+					} else  {
 						show_eval(b,lastv,edepth);
-						state->converged_record[state->position.move_num] = last_move; 
-					}
+					} 
 				}
-			}
 #endif
+			}
 		}
-		
+
 		depth++;
+		
+		if (!pondering && (small_window || ABS(v.v) > FORCED_WIN || is_zero_move(&last_move))) { 
+			state->converged_record[state->position.move_num] = *move; 
+		} else {
+			state->converged_record[state->position.move_num] = last_move;
+		}
+
+		last_move = *move;
 #if USE_PBRAIN
-		if (!pondering && mulling && depth >= 6) 
-			brain_insert(b, move);
+		if (mull_stage == 2)
+			tdepth = state->current_depth;
+		else 
+			tdepth = 6;
+		if (!pondering && mulling && ((depth >= tdepth && (!terminate || !state->converged)) 
+					      || ABS(v.v) > FORCED_WIN)) {
+			int tmp = brain_insert(b, move);
+			if (!tmp)
+				state->current_depth = depth;
+		}
 #endif
 		
 	}
@@ -2126,6 +2219,7 @@ int make_move(Position *b, Move *move)
 {
 	Position b1;
 	Eval bestv, v1;
+	int result;
 
 	if (next_to_play(b) == 1)
 		b->flags |= FLAG_COMPUTER_WHITE;
@@ -2139,11 +2233,15 @@ int make_move(Position *b, Move *move)
 	zero_move(move);
 
 	init_hash_table();
+	brain_fill_hash(NULL);
 
 	if (find_winner(b, NULL))
 		return 0;
+
 	regen_moves(b);
+
 	v1 = eval(b, INFINITY, MAX_DEPTH);
+
 	estimate_game_stage(b,1);
 
 	b1 = (*b);
@@ -2153,13 +2251,12 @@ int make_move(Position *b, Move *move)
 	lprintf(0,"searching position: %s\n", position_to_ppn(b));
 
 #if USE_PBRAIN
-	if (!state->auto_exit)
-		if (brain_lookup(b, move, &bestv, 0)) {
-			state->converged_record[state->position.move_num] = *move; 
-			goto got_move;
-		} else {
-			brain_fill_hash(&b1);
-		}	
+	if ((result = brain_lookup(b, move, &bestv, 0)) > 0) {
+		state->converged_record[state->position.move_num] = *move; 
+		if (result == 1)
+			state->last_book_move = state->position.move_num;
+		goto got_move;
+	} 
 #endif
 
 	bestv = search(&b1, move);
@@ -2231,8 +2328,6 @@ void ponder_move(Position *b, Move *move)
 
 	zero_move(&m1);
 
-	brain_fill_hash(&b1);
-	
 	if (!check_hash(&b1, 0, makeeval(&b1, ILLEGAL), &v1, &m1) || 
 	    is_zero_move(&m1)) {
 		lprintf(0,"searching for move to ponder\n");
@@ -2252,16 +2347,13 @@ void ponder_move(Position *b, Move *move)
 		return;
 	}
 
-	if (!state->auto_exit)
-		if (brain_lookup(&b1, &m1, &bestv, 1)) {
-			zero_move(move);
-			return;		
-		}
+	if (brain_lookup(&b1, &m1, &bestv, 1) > 0) {
+		zero_move(move);
+		return;		
+	}
 	
 	timer_start(next_to_play(&b1));
 
-	brain_fill_hash(&b1);
-	
 	pondering = 1;
 	v1 = eval(&b1, INFINITY, MAX_DEPTH);
 
@@ -2300,3 +2392,4 @@ void ponder_move(Position *b, Move *move)
 }
 
 
+
